/* Test of element extractors */

#include <stdlib.h>

#include <testlib/testlib.h>

#include "matvec/macros.h"

#include "macros.h"

/* quiet -Werror=missing-prototypes */
void test_MATVEC_MAT3X3_AS_MDARRAY( void );
void test_MATVEC_MAT4X4_AS_MDARRAY( void );
void test_MATVEC_MAT3X3_AS_ARRAY( void );
void test_MATVEC_MAT4X4_AS_ARRAY( void );
void test_MATVEC_VEC_AS_ARRAY( void );
void test_MATVEC_VEC_AS_STRUCT( void );

void
test_MATVEC_MAT3X3_AS_MDARRAY( void )
{
  def_MAT3X3_AS_MDARRAY;

  TEST( "MATVEC_MAT3X3_AS_MDARRAY", 1,
	T3X3EQ(00, 01, 02,
	       10, 11, 12,
	       20, 21, 22,
	       MATVEC_MAT3X3_AS_MDARRAY(mat) )
	);
}

void
test_MATVEC_MAT4X4_AS_MDARRAY( void )
{
  def_MAT4X4_AS_MDARRAY;
  
  TEST( "MATVEC_MAT4X4_AS_MDARRAY", 1,
	T4X4EQ(00, 01, 02, 03,
	       10, 11, 12, 13,
	       20, 21, 22, 23,
	       30, 31, 32, 33,
	       MATVEC_MAT4X4_AS_MDARRAY(mat) )
	);
}

void
test_MATVEC_MAT3X3_AS_ARRAY( void )
{
  def_MAT3X3_AS_ARRAY;
  
  TEST( "MATVEC_MAT3X3_AS_ARRAY", 1,
	T3X3EQ(00, 01, 02,
	       10, 11, 12,
	       20, 21, 22,
	       MATVEC_MAT3X3_AS_ARRAY(mat, 3) )
	);
}

void
test_MATVEC_MAT4X4_AS_ARRAY( void )
{
  def_MAT4X4_AS_ARRAY;

  TEST( "MATVEC_MAT4X4_AS_ARRAY", 1,
	T4X4EQ( 00, 01, 02, 03,
		10, 11, 12, 13,
		20, 21, 22, 23,
		30, 31, 32, 33,
		MATVEC_MAT4X4_AS_ARRAY(mat, 4) )
	);
}

void
test_MATVEC_VEC_AS_ARRAY( void )
{
  MATTYPE vec0[3] = { 0, 1, 2 };
  MATTYPE vec1[6] = { 0, 9, 1, 9, 2, 9 };

  TEST( "MATVEC_VEC_AS_ARRAY(stride=1)",
	TVECEQ( 0, 1, 2, MATVEC_VEC_AS_ARRAY(vec0,1)), 1);

  TEST( "MATVEC_VEC_AS_ARRAY(stride=2)",
	TVECEQ( 0, 1, 2, MATVEC_VEC_AS_ARRAY(vec1,2)), 1);
}

void
test_MATVEC_VEC_AS_STRUCT( void )
{
  struct { MATTYPE a; MATTYPE b; MATTYPE c; } vec = { 0, 1, 2 };
  
  TEST( "MATVEC_VEC_AS_STRUCT",
	TVECEQ( 0, 1, 2, MATVEC_VEC_AS_STRUCT(vec, a, b, c)), 1);
}

int main( void )
{
  START( "matvec extractors" );

  test_MATVEC_MAT3X3_AS_MDARRAY( );
  test_MATVEC_MAT4X4_AS_MDARRAY( );
  test_MATVEC_MAT3X3_AS_ARRAY( );
  test_MATVEC_MAT4X4_AS_ARRAY( );
  test_MATVEC_VEC_AS_ARRAY( );
  test_MATVEC_VEC_AS_STRUCT( );

  SUMMARY();

  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
