/* Test of vector transformations */

#include <stdlib.h>
#include <math.h>

#include <testlib/testlib.h>

#include "matvec/macros.h"

#include "macros.h"

#define VEC(vector) MATVEC_VEC_AS_ARRAY(vector,1)
#define MAT(matrix) MATVEC_MAT3X3_AS_ARRAY(matrix,3)

/* quiet -Werror=missing-prototypes */
void test_MATVEC_VEC_ROTATE_TRANSLATE( void );
void test_MATVEC_VEC_TRANSLATE_ROTATE( void );

void test_MATVEC_VEC_ROTATE_TRANSLATE( void )
{
  double theta = DEG2RAD(30);
  
  MATTYPE rot[9];
  MATTYPE vtrans[3] = { 0, 0, 0 };
  MATTYPE vx[3]    = { 1, 0, 0 };
  MATTYPE vz[3]    = { 0, 0, 1 };
  MATTYPE vout[3];
  MATTYPE vexp[3] = { 0, 0, 0 };

  /* No Translation */

  /* rotate X towards Y.  vector rotates from Y towards X. */
  MATVEC_MAT3X3_ROT_Z(MAT(rot), theta);
  MATVEC_VEC_ROTATE_TRANSLATE( VEC(vout), VEC(vx), MAT(rot), VEC(vtrans) );

  vexp[0] = COS30;
  vexp[1] = -SIN30;
  vexp[2] = 0;

  TEST( "MATVEC_VEC_ROTATE_TRANSLATE: Z rot ", 1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Z towards X.  vector rotates from X towards Z */
  MATVEC_MAT3X3_ROT_Y(MAT(rot), theta);
  MATVEC_VEC_ROTATE_TRANSLATE( VEC(vout), VEC(vx), MAT(rot), VEC(vtrans) );

  vexp[0] = COS30;
  vexp[1] = 0;
  vexp[2] = SIN30;

  TEST( "MATVEC_VEC_ROTATE_TRANSLATE: Y rot ", 1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Y towards Z.  vector rotates from Z towards Y */
  MATVEC_MAT3X3_ROT_X(MAT(rot), theta);
  MATVEC_VEC_ROTATE_TRANSLATE( VEC(vout), VEC(vz), MAT(rot), VEC(vtrans) );

  vexp[0] = 0;
  vexp[1] = SIN30;
  vexp[2] = COS30;

  TEST( "MATVEC_VEC_ROTATE_TRANSLATE: X rot ", 1, TVECAR_FPEQ( vout, vexp ));

  /*****************************************************/

  /* Translation */

  vtrans[0] = 22;
  vtrans[1] = -19;
  vtrans[2] = -30;

  /* rotate X towards Y.  vector rotates from Y towards X. */
  MATVEC_MAT3X3_ROT_Z(MAT(rot), theta);
  MATVEC_VEC_ROTATE_TRANSLATE( VEC(vout), VEC(vx), MAT(rot), VEC(vtrans) );

  vexp[0] =  COS30 + vtrans[0];
  vexp[1] = -SIN30 + vtrans[1];
  vexp[2] =      0 + vtrans[2];

  TEST( "MATVEC_VEC_ROTATE_TRANSLATE: Z rot + trans",
	1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Z towards X.  vector rotates from X towards Z */
  MATVEC_MAT3X3_ROT_Y(MAT(rot), theta);
  MATVEC_VEC_ROTATE_TRANSLATE( VEC(vout), VEC(vx), MAT(rot), VEC(vtrans) );

  vexp[0] = COS30 + vtrans[0];
  vexp[1] =     0 + vtrans[1];
  vexp[2] = SIN30 + vtrans[2];

  TEST( "MATVEC_VEC_ROTATE_TRANSLATE: Y rot + trans",
	1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Y towards Z.  vector rotates from Z towards Y */
  MATVEC_MAT3X3_ROT_X(MAT(rot), theta);
  MATVEC_VEC_ROTATE_TRANSLATE( VEC(vout), VEC(vz), MAT(rot), VEC(vtrans) );

  vexp[0] =     0 + vtrans[0];
  vexp[1] = SIN30 + vtrans[1];
  vexp[2] = COS30 + vtrans[2];

  TEST( "MATVEC_VEC_ROTATE_TRANSLATE: X rot + trans",
	1, TVECAR_FPEQ( vout, vexp ));

}

void test_MATVEC_VEC_TRANSLATE_ROTATE( void )
{
  double theta = DEG2RAD(30);
  
  MATTYPE rot[9];
  MATTYPE vtrans[3] = { 0, 0, 0 };
  MATTYPE vx[3]    = { 1, 0, 0 };
  MATTYPE vz[3]    = { 0, 0, 1 };
  MATTYPE vin[3];
  MATTYPE vout[3];
  MATTYPE vexp[3] = { 0, 0, 0 };

  /* No Translation */

  /* rotate X towards Y.  vector rotates from Y towards X. */
  MATVEC_MAT3X3_ROT_Z(MAT(rot), theta);
  MATVEC_VEC_TRANSLATE_ROTATE( MATTYPE, VEC(vout),
			       VEC(vx), VEC(vtrans), MAT(rot) );

  vexp[0] = COS30;
  vexp[1] = -SIN30;
  vexp[2] = 0;

  TEST( "MATVEC_VEC_TRANSLATE_ROTATE: Z rot ", 1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Z towards X.  vector rotates from X towards Z */
  MATVEC_MAT3X3_ROT_Y(MAT(rot), theta);
  MATVEC_VEC_TRANSLATE_ROTATE( MATTYPE, VEC(vout),
			       VEC(vx), VEC(vtrans), MAT(rot) );

  vexp[0] = COS30;
  vexp[1] = 0;
  vexp[2] = SIN30;

  TEST( "MATVEC_VEC_TRANSLATE_ROTATE: Y rot ", 1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Y towards Z.  vector rotates from Z towards Y */
  MATVEC_MAT3X3_ROT_X(MAT(rot), theta);
  MATVEC_VEC_TRANSLATE_ROTATE( MATTYPE, VEC(vout),
			       VEC(vz), VEC(vtrans), MAT(rot) );

  vexp[0] = 0;
  vexp[1] = SIN30;
  vexp[2] = COS30;

  TEST( "MATVEC_VEC_TRANSLATE_ROTATE: X rot ", 1, TVECAR_FPEQ( vout, vexp ));

  /*****************************************************/

  /* Translation */

  vtrans[0] = 22;
  vtrans[1] = -19;
  vtrans[2] = -30;

  
  MATVEC_VEC_SUB( VEC(vin), VEC(vx), VEC(vtrans) );


  /* rotate X towards Y.  vector rotates from Y towards X. */
  MATVEC_MAT3X3_ROT_Z(MAT(rot), theta);
  MATVEC_VEC_TRANSLATE_ROTATE( MATTYPE, VEC(vout),
			       VEC(vin), VEC(vtrans), MAT(rot) );

  vexp[0] =  COS30;
  vexp[1] = -SIN30;
  vexp[2] =      0;

  TEST( "MATVEC_VEC_TRANSLATE_ROTATE: Z rot + trans",
	1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Z towards X.  vector rotates from X towards Z */
  MATVEC_MAT3X3_ROT_Y(MAT(rot), theta);
  MATVEC_VEC_TRANSLATE_ROTATE( MATTYPE, VEC(vout),
			       VEC(vin), VEC(vtrans), MAT(rot) );

  vexp[0] = COS30;
  vexp[1] =     0;
  vexp[2] = SIN30;

  TEST( "MATVEC_VEC_TRANSLATE_ROTATE: Y rot + trans",
	1, TVECAR_FPEQ( vout, vexp ));

  /***************************/

  /* rotate Y towards Z.  vector rotates from Z towards Y */

  MATVEC_VEC_SUB( VEC(vin), VEC(vz), VEC(vtrans) );

  MATVEC_MAT3X3_ROT_X(MAT(rot), theta);
  MATVEC_VEC_TRANSLATE_ROTATE( MATTYPE, VEC(vout),
			       VEC(vin), VEC(vtrans), MAT(rot) );

  vexp[0] =     0;
  vexp[1] = SIN30;
  vexp[2] = COS30;

  TEST( "MATVEC_VEC_TRANSLATE_ROTATE: X rot + trans",
	1, TVECAR_FPEQ( vout, vexp ));

}

int main( void )
{
  START( "Vector Operations" );

  test_MATVEC_VEC_ROTATE_TRANSLATE();
  test_MATVEC_VEC_TRANSLATE_ROTATE();

  SUMMARY();

  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
