/* Test of vector operations */

#include <stdlib.h>
#include <math.h>

#include <testlib/testlib.h>

#include "matvec/macros.h"

#include "macros.h"

#define MAT4X4(matrix) MATVEC_MAT4X4_AS_ARRAY(matrix,4)
#define MAT3X3(matrix) MATVEC_MAT3X3_AS_ARRAY(matrix,3)

/* quiet -Werror=missing-prototypes */
void test_MATVEC_MAT3X3_ROT_X( void );
void test_MATVEC_MAT3X3_ROT_Y( void );
void test_MATVEC_MAT3X3_ROT_Z( void );
void test_MATVEC_MAT3X3_ROT_YMXZ( void );
void test_MATVEC_MAT3X3_ROT_ZXZ( void );


void test_MATVEC_MAT3X3_ROT_X( void )
{
  double theta = DEG2RAD(30);
  MATTYPE mat[9];
  MATTYPE exp[9] = { 1,      0,     0,
		     0,  COS30, SIN30,
		     0, -SIN30, COS30 };

  MATVEC_MAT3X3_ROT_X( MAT3X3(mat), theta );

  TEST( "MATVEC_MAT3X3_ROT_X", 1, T3X3AR_FPEQ( mat, exp ) );
}

void test_MATVEC_MAT3X3_ROT_Y( void )
{
  double theta = DEG2RAD(30);
  MATTYPE mat[9];
  MATTYPE exp[9] = { COS30,   0, -SIN30,
		         0,   1,      0,
		     SIN30,   0,  COS30 };

  MATVEC_MAT3X3_ROT_Y( MAT3X3(mat), theta );

  TEST( "MATVEC_MAT3X3_ROT_Y", 1, T3X3AR_FPEQ( mat, exp ) );
}

void test_MATVEC_MAT3X3_ROT_Z( void )
{
  double theta = DEG2RAD(30);
  MATTYPE mat[9];
  MATTYPE exp[9] = {  COS30, SIN30, 0,
		     -SIN30, COS30, 0,
		          0,     0, 1 };

  MATVEC_MAT3X3_ROT_Z( MAT3X3(mat), theta );

  TEST( "MATVEC_MAT3X3_ROT_Z", 1, T3X3AR_FPEQ( mat, exp ) );
}

void test_MATVEC_MAT3X3_ROT_YMXZ( void )
{
  double azimuth   = DEG2RAD(30);
  double elevation = DEG2RAD(45);
  double clocking  = DEG2RAD(60);

  MATTYPE a[9];
  MATTYPE b[9];
  MATTYPE c[9];
  MATTYPE d[9];
  MATTYPE t[9];

  MATTYPE aec[9];

  MATVEC_MAT3X3_ROT_Y(MAT3X3(d), azimuth);
  MATVEC_MAT3X3_ROT_X(MAT3X3(c), -elevation);
  MATVEC_MAT3X3_ROT_Z(MAT3X3(b), clocking);

  MATVEC_MAT3X3_MULT( MAT3X3(t), MAT3X3(b), MAT3X3(c) );
  MATVEC_MAT3X3_MULT( MAT3X3(a), MAT3X3(t), MAT3X3(d) );

  MATVEC_MAT3X3_ROT_YMXZ( MAT3X3(aec), azimuth, elevation, clocking );

  TEST( "MATVEC_MAT3X3_ROT_YMXZ", 1, T3X3AR_FPEQ(a, aec));
}

void test_MATVEC_MAT3X3_ROT_ZXZ( void )
{
  double phi   = DEG2RAD(30);
  double theta = DEG2RAD(45);
  double psi   = DEG2RAD(60);

  /* A = BCD */
  MATTYPE A[9];
  MATTYPE B[9];
  MATTYPE C[9];
  MATTYPE D[9];

  MATTYPE t[9];


  MATVEC_MAT3X3_ROT_Z(MAT3X3(D), phi);
  MATVEC_MAT3X3_ROT_X(MAT3X3(C), theta);
  MATVEC_MAT3X3_ROT_Z(MAT3X3(B), psi);

  MATVEC_MAT3X3_MULT( MAT3X3(t), MAT3X3(B), MAT3X3(C));
  MATVEC_MAT3X3_MULT( MAT3X3(A), MAT3X3(t), MAT3X3(D));

  MATVEC_MAT3X3_ROT_ZXZ( MAT3X3(t), phi, theta, psi );

  TEST( "MATVEC_MAT3X3_ROT_ZXZ", 1, T3X3AR_FPEQ(A, t) );
}


int main( void )
{
  START( "Transformation Matrix Operations" );

  test_MATVEC_MAT3X3_ROT_X();
  test_MATVEC_MAT3X3_ROT_Y();
  test_MATVEC_MAT3X3_ROT_Z();
  test_MATVEC_MAT3X3_ROT_YMXZ();
  test_MATVEC_MAT3X3_ROT_ZXZ();

  SUMMARY();

  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}


